@@ -48,7 +48,9 @@ struct BuilderContext
   SoupSession    *soup_session;
   CURL           *curl_session;
   char           *arch;
+  char           *default_branch;
   char           *stop_at;
+  gint64          source_date_epoch;
 
   GFile          *download_dir;
   GPtrArray      *sources_dirs;
@@ -118,6 +120,7 @@ builder_context_finalize (GObject *object)
   g_clear_object (&self->options);
   g_clear_object (&self->sdk_config);
   g_free (self->arch);
+  g_free (self->default_branch);
   g_free (self->state_subdir);
   g_free (self->stop_at);
   g_strfreev (self->cleanup);
@@ -565,6 +568,27 @@ builder_context_set_arch (BuilderContext *self,
   self->arch = g_strdup (arch);
 }
 
+const char *
+builder_context_get_default_branch (BuilderContext *self)
+{
+  return (const char *) self->default_branch;
+}
+
+void
+builder_context_set_default_branch (BuilderContext *self,
+                                    const char     *default_branch)
+{
+  g_free (self->default_branch);
+  self->default_branch = g_strdup (default_branch);
+}
+
+void
+builder_context_set_source_date_epoch (BuilderContext *self,
+                                       gint64 source_date_epoch)
+{
+  self->source_date_epoch = source_date_epoch;
+}
+
 const char *
 builder_context_get_stop_at (BuilderContext *self)
 {
@@ -1005,8 +1029,21 @@ builder_context_set_enable_ccache (BuilderContext *self,
 }
 
 char **
-builder_context_extend_env (BuilderContext *self,
-                            char          **envp)
+builder_context_extend_env_pre (BuilderContext *self,
+                                char          **envp)
+{
+  if (self->source_date_epoch != 0)
+    {
+      g_autofree char *s_d_e = g_strdup_printf ("%" G_GUINT64_FORMAT, self->source_date_epoch);
+      envp = g_environ_setenv (envp, "SOURCE_DATE_EPOCH", s_d_e, FALSE);
+    }
+
+  return envp;
+}
+
+char **
+builder_context_extend_env_post (BuilderContext *self,
+                                 char          **envp)
 {
   g_autofree char *path = NULL;
   const char *ccache_dir = NULL;
