@@ -49,12 +49,6 @@ gboolean builder_git_checkout           (const char      *repo_location,
                                          GFile           *dest,
                                          BuilderContext  *context,
                                          GError         **error);
-gboolean builder_git_checkout_dir       (const char      *repo_location,
-                                         const char      *branch,
-                                         const char      *dir,
-                                         GFile           *dest,
-                                         BuilderContext  *context,
-                                         GError         **error);
 gboolean builder_git_shallow_mirror_ref (const char     *repo_location,
                                          const char     *destination_path,
                                          const char     *ref,
