@@ -74,6 +74,8 @@ static char *opt_gpg_homedir;
 static char **opt_key_ids;
 static char **opt_sources_dirs;
 static char **opt_sources_urls;
+static char **opt_add_tags;
+static char **opt_remove_tags;
 static int opt_jobs;
 static char *opt_mirror_screenshots_url;
 static char *opt_install_deps_from;
@@ -89,6 +91,8 @@ static GOptionEntry entries[] = {
   { "version", 0, 0, G_OPTION_ARG_NONE, &opt_version, "Print version information and exit", NULL },
   { "arch", 0, 0, G_OPTION_ARG_STRING, &opt_arch, "Architecture to build for (must be host compatible)", "ARCH" },
   { "default-branch", 0, 0, G_OPTION_ARG_STRING, &opt_default_branch, "Change the default branch", "BRANCH" },
+  { "add-tag", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_add_tags, "Add a tag to the build", "TAG"},
+  { "remove-tag", 0, 0, G_OPTION_ARG_STRING_ARRAY, &opt_remove_tags, "Remove a tag from the build", "TAG"},
   { "run", 0, 0, G_OPTION_ARG_NONE, &opt_run, "Run a command in the build directory (see --run --help)", NULL },
   { "ccache", 0, 0, G_OPTION_ARG_NONE, &opt_ccache, "Use ccache", NULL },
   { "disable-cache", 0, 0, G_OPTION_ARG_NONE, &opt_disable_cache, "Disable cache lookups", NULL },
@@ -252,6 +256,46 @@ do_export (BuilderContext *build_context,
   return TRUE;
 }
 
+static gboolean
+flatpak_version_check (int major,
+                       int minor,
+                       int micro)
+{
+  static int flatpak_major = 0;
+  static int flatpak_minor = 0;
+  static int flatpak_micro = 0;
+
+  if (flatpak_major == 0 &&
+      flatpak_minor == 0 &&
+      flatpak_micro == 0)
+    {
+      const char * argv[] = { "flatpak", "--version", NULL };
+      g_autoptr(GSubprocess) subp = NULL;
+      g_autofree char *out = NULL;
+ 
+      subp = g_subprocess_newv (argv, G_SUBPROCESS_FLAGS_STDOUT_PIPE, NULL);
+      g_subprocess_communicate_utf8 (subp, NULL, NULL, &out, NULL, NULL);
+
+      if (sscanf (out, "Flatpak %d.%d.%d", &flatpak_major, &flatpak_minor, &flatpak_micro) != 3)
+        g_warning ("Failed to get flatpak version");
+
+      g_debug ("Using Flatpak version %d.%d.%d", flatpak_major, flatpak_minor, flatpak_micro);
+    }
+
+  if (flatpak_major > major)
+    return TRUE;
+  if (flatpak_major < major)
+    return FALSE;
+  if (flatpak_minor > minor)
+    return TRUE;
+  if (flatpak_minor < minor)
+    return FALSE;
+  if (flatpak_micro >= micro)
+    return TRUE;
+
+  return FALSE;
+}
+
 static gboolean
 do_install (BuilderContext *build_context,
             const gchar    *repodir,
@@ -276,7 +320,8 @@ do_install (BuilderContext *build_context,
     g_ptr_array_add (args, g_strdup ("--system"));
 
   g_ptr_array_add (args, g_strdup ("-y"));
-
+  if (flatpak_version_check (1, 2, 0))
+    g_ptr_array_add (args, g_strdup ("--noninteractive"));
   g_ptr_array_add (args, g_strdup ("--reinstall"));
 
   ref = flatpak_build_untyped_ref (id, branch,
@@ -401,6 +446,7 @@ main (int    argc,
   int i, first_non_arg, orig_argc;
   int argnr;
   char *p;
+  struct stat statbuf;
 
   setlocale (LC_ALL, "");
 
@@ -417,7 +463,6 @@ main (int    argc,
   else
     g_unsetenv ("GIO_USE_VFS");
 
-
   /* Work around libsoup/glib race condition, as per:
      https://bugzilla.gnome.org/show_bug.cgi?id=796031 and
      https://bugzilla.gnome.org/show_bug.cgi?id=674885#c87 */
@@ -582,21 +627,27 @@ main (int    argc,
 
       cleanup_manifest_dir = g_object_ref (build_subdir);
 
+      int mirror_flags = FLATPAK_GIT_MIRROR_FLAGS_MIRROR_SUBMODULES;
+
+      if (opt_disable_updates)
+        {
+          mirror_flags |= FLATPAK_GIT_MIRROR_FLAGS_UPDATE;
+	}
+
       if (!builder_git_mirror_repo (opt_from_git,
                                     NULL,
-                                    opt_disable_updates?0:FLATPAK_GIT_MIRROR_FLAGS_UPDATE,
+                                    mirror_flags,
                                     git_branch, build_context, &error))
         {
           g_printerr ("Can't clone manifest repo: %s\n", error->message);
           return 1;
         }
 
-      if (!builder_git_checkout_dir (opt_from_git,
-                                     git_branch,
-                                     manifest_dirname,
-                                     build_subdir,
-                                     build_context,
-                                     &error))
+      if (!builder_git_checkout (opt_from_git,
+                                 git_branch,
+                                 build_subdir,
+                                 build_context,
+                                 &error))
         {
           g_printerr ("Can't check out manifest repo: %s\n", error->message);
           return 1;
@@ -619,6 +670,9 @@ main (int    argc,
       return 1;
     }
 
+  if (stat (flatpak_file_get_path_cached (manifest_file), &statbuf) == 0)
+    builder_context_set_source_date_epoch (build_context, (gint64)statbuf.st_mtime);
+
   manifest_sha256 = g_compute_checksum_for_string (G_CHECKSUM_SHA256, manifest_contents, -1);
 
   if (opt_skip_if_unchanged)
@@ -645,8 +699,14 @@ main (int    argc,
       return 1;
     }
 
+  if (opt_remove_tags)
+    builder_manifest_remove_tags (manifest, (const char **)opt_remove_tags);
+
+  if (opt_add_tags)
+    builder_manifest_add_tags (manifest, (const char **)opt_add_tags);
+
   if (opt_default_branch)
-    builder_manifest_set_default_branch (manifest, opt_default_branch);
+    builder_context_set_default_branch (build_context, opt_default_branch);
 
   if (opt_collection_id)
     builder_manifest_set_default_collection_id (manifest, opt_collection_id);
@@ -664,7 +724,7 @@ main (int    argc,
     {
       if (!builder_manifest_show_deps (manifest, build_context, &error))
         {
-          g_printerr ("Error running %s: %s\n", argv[3], error->message);
+          g_printerr ("Error calculating deps: %s\n", error->message);
           return 1;
         }
 
@@ -676,7 +736,7 @@ main (int    argc,
       if (!builder_manifest_install_deps (manifest, build_context, opt_install_deps_from, opt_user, opt_installation,
                                           opt_yes, &error))
         {
-          g_printerr ("Error running %s: %s\n", argv[3], error->message);
+          g_printerr ("Error installing deps: %s\n", error->message);
           return 1;
         }
       if (opt_install_deps_only)
@@ -884,7 +944,7 @@ main (int    argc,
   if (opt_mirror_screenshots_url && !opt_export_only)
     {
       g_autofree char *screenshot_subdir = g_strdup_printf ("%s-%s", builder_manifest_get_id (manifest),
-                                                            builder_manifest_get_branch (manifest));
+                                                            builder_manifest_get_branch (manifest, build_context));
       g_autofree char *url = g_build_filename (opt_mirror_screenshots_url, screenshot_subdir, NULL);
       g_autofree char *xml_relpath = g_strdup_printf ("files/share/app-info/xmls/%s.xml.gz", builder_manifest_get_id (manifest));
       g_autoptr(GFile) xml = g_file_resolve_relative_path (app_dir, xml_relpath);
@@ -966,7 +1026,7 @@ main (int    argc,
       if (!do_export (build_context, &error,
                       FALSE,
                       flatpak_file_get_path_cached (export_repo),
-                      app_dir_path, exclude_dirs, builder_manifest_get_branch (manifest),
+                      app_dir_path, exclude_dirs, builder_manifest_get_branch (manifest, build_context),
                       builder_manifest_get_collection_id (manifest),
                       "--exclude=/lib/debug/*",
                       "--include=/lib/debug/app",
@@ -1000,7 +1060,7 @@ main (int    argc,
                                    "/share/runtime/locale/", NULL);
           if (!do_export (build_context, &error, TRUE,
                           flatpak_file_get_path_cached (export_repo),
-                          app_dir_path, NULL, builder_manifest_get_branch (manifest),
+                          app_dir_path, NULL, builder_manifest_get_branch (manifest, build_context),
                           builder_manifest_get_collection_id (manifest),
                           metadata_arg,
                           files_arg,
@@ -1020,7 +1080,7 @@ main (int    argc,
 
           if (!do_export (build_context, &error, TRUE,
                           flatpak_file_get_path_cached (export_repo),
-                          app_dir_path, NULL, builder_manifest_get_branch (manifest),
+                          app_dir_path, NULL, builder_manifest_get_branch (manifest, build_context),
                           builder_manifest_get_collection_id (manifest),
                           "--metadata=metadata.debuginfo",
                           builder_context_get_build_runtime (build_context) ? "--files=usr/lib/debug" : "--files=files/lib/debug",
@@ -1051,7 +1111,7 @@ main (int    argc,
 
           if (!do_export (build_context, &error, TRUE,
                           flatpak_file_get_path_cached (export_repo),
-                          app_dir_path, NULL, builder_manifest_get_branch (manifest),
+                          app_dir_path, NULL, builder_manifest_get_branch (manifest, build_context),
                           builder_manifest_get_collection_id (manifest),
                           metadata_arg, files_arg,
                           NULL))
@@ -1070,7 +1130,7 @@ main (int    argc,
 
           if (!do_export (build_context, &error, TRUE,
                           flatpak_file_get_path_cached (export_repo),
-                          app_dir_path, NULL, builder_manifest_get_branch (manifest),
+                          app_dir_path, NULL, builder_manifest_get_branch (manifest, build_context),
                           builder_manifest_get_collection_id (manifest),
                           "--metadata=metadata.sources",
                           "--files=sources",
@@ -1090,7 +1150,7 @@ main (int    argc,
 
           if (!do_export (build_context, &error, TRUE,
                           flatpak_file_get_path_cached (export_repo),
-                          app_dir_path, NULL, builder_manifest_get_branch (manifest),
+                          app_dir_path, NULL, builder_manifest_get_branch (manifest, build_context),
                           builder_manifest_get_collection_id (manifest),
                           "--metadata=metadata.platform",
                           "--files=platform",
@@ -1124,7 +1184,7 @@ main (int    argc,
           files_arg = g_strconcat ("--files=platform/share/runtime/locale/", NULL);
           if (!do_export (build_context, &error, TRUE,
                           flatpak_file_get_path_cached (export_repo),
-                          app_dir_path, NULL, builder_manifest_get_branch (manifest),
+                          app_dir_path, NULL, builder_manifest_get_branch (manifest, build_context),
                           builder_manifest_get_collection_id (manifest),
                           metadata_arg,
                           files_arg,
@@ -1144,7 +1204,7 @@ main (int    argc,
         g_printerr ("NOTE: No export due to --require-changes, ignoring --install\n");
       else if (!do_install (build_context, flatpak_file_get_path_cached (export_repo),
                             builder_manifest_get_id (manifest),
-                            builder_manifest_get_branch (manifest),
+                            builder_manifest_get_branch (manifest, build_context),
                             &error))
         {
           g_printerr ("Install failed: %s\n", error->message);
