@@ -39,6 +39,16 @@
 #include "builder-flatpak-utils.h"
 #include "builder-utils.h"
 
+G_DEFINE_QUARK (builder-curl-error, builder_curl_error)
+G_DEFINE_QUARK (builder-yaml-parse-error, builder_yaml_parse_error)
+
+#ifdef FLATPAK_BUILDER_ENABLE_YAML
+#include <yaml.h>
+
+G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (yaml_parser_t, yaml_parser_delete)
+G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC (yaml_document_t, yaml_document_delete)
+#endif
+
 char *
 builder_uri_to_filename (const char *uri)
 {
@@ -325,7 +335,7 @@ migrate_locale_dir (GFile      *source_dir,
 
           if (!flatpak_cp_a (child, locale_subdir,
                              FLATPAK_CP_FLAGS_MERGE | FLATPAK_CP_FLAGS_MOVE,
-                             NULL, error))
+                             NULL, NULL, error))
             return FALSE;
 
           target = g_build_filename ("../../share/runtime/locale", relative, NULL);
@@ -367,6 +377,155 @@ builder_migrate_locale_dirs (GFile   *root_dir,
   return TRUE;
 }
 
+#ifdef FLATPAK_BUILDER_ENABLE_YAML
+
+static JsonNode *
+parse_yaml_node_to_json (yaml_document_t *doc, yaml_node_t *node)
+{
+  JsonNode *json = json_node_alloc ();
+  const char *scalar = NULL;
+  g_autoptr(JsonArray) array = NULL;
+  g_autoptr(JsonObject) object = NULL;
+  yaml_node_item_t *item = NULL;
+  yaml_node_pair_t *pair = NULL;
+
+  switch (node->type)
+    {
+    case YAML_NO_NODE:
+      json_node_init_null (json);
+      break;
+    case YAML_SCALAR_NODE:
+      scalar = (gchar *) node->data.scalar.value;
+      if (node->data.scalar.style == YAML_PLAIN_SCALAR_STYLE)
+        {
+          if (strcmp (scalar, "true") == 0)
+            {
+              json_node_init_boolean (json, TRUE);
+              break;
+            }
+          else if (strcmp (scalar, "false") == 0)
+            {
+              json_node_init_boolean (json, FALSE);
+              break;
+            }
+          else if (strcmp (scalar, "null") == 0)
+            {
+              json_node_init_null (json);
+              break;
+            }
+
+          if (*scalar != '\0')
+            {
+              gchar *endptr;
+              gint64 num = g_ascii_strtoll (scalar, &endptr, 10);
+              if (*endptr == '\0')
+                {
+                  json_node_init_int (json, num);
+                  break;
+                }
+              else if (*endptr == '.')
+                {
+                  g_ascii_strtoll (endptr + 1, &endptr, 10);
+                  if (*endptr == '\0')
+                    g_warning ("%zu:%zu: '%s' will be parsed as a number by many YAML parsers",
+                               node->start_mark.line + 1, node->start_mark.column + 1, scalar);
+                }
+            }
+        }
+
+      json_node_init_string (json, scalar);
+      break;
+    case YAML_SEQUENCE_NODE:
+      array = json_array_new ();
+      for (item = node->data.sequence.items.start; item < node->data.sequence.items.top; item++)
+        {
+          yaml_node_t *child = yaml_document_get_node (doc, *item);
+          if (child != NULL)
+            json_array_add_element (array, parse_yaml_node_to_json (doc, child));
+        }
+
+      json_node_init_array (json, array);
+      break;
+    case YAML_MAPPING_NODE:
+      object = json_object_new ();
+      for (pair = node->data.mapping.pairs.start; pair < node->data.mapping.pairs.top; pair++)
+        {
+          yaml_node_t *key = yaml_document_get_node (doc, pair->key);
+          yaml_node_t *value = yaml_document_get_node (doc, pair->value);
+
+          g_warn_if_fail (key->type == YAML_SCALAR_NODE);
+          json_object_set_member (object, (gchar *) key->data.scalar.value,
+                                  parse_yaml_node_to_json (doc, value));
+        }
+
+      json_node_init_object (json, object);
+      break;
+    }
+
+  return json;
+}
+
+static JsonNode *
+parse_yaml_to_json (const gchar *contents,
+                    GError      **error)
+{
+  if (error)
+    *error = NULL;
+
+  g_auto(yaml_parser_t) parser = {0};
+  g_auto(yaml_document_t) doc = {{0}};
+
+  if (!yaml_parser_initialize (&parser))
+    g_error ("yaml_parser_initialize is out of memory.");
+  yaml_parser_set_input_string (&parser, (yaml_char_t *) contents, strlen (contents));
+
+  if (!yaml_parser_load (&parser, &doc))
+    {
+      g_set_error (error, BUILDER_YAML_PARSE_ERROR, parser.error, "%zu:%zu: %s", parser.problem_mark.line + 1,
+                   parser.problem_mark.column + 1, parser.problem);
+      return NULL;
+    }
+
+  yaml_node_t *root = yaml_document_get_root_node (&doc);
+  if (root == NULL)
+    {
+      g_set_error (error, BUILDER_YAML_PARSE_ERROR, YAML_PARSER_ERROR,
+                   "Document has no root node.");
+      return NULL;
+    }
+
+  return parse_yaml_node_to_json (&doc, root);
+}
+
+#else // FLATPAK_BUILDER_ENABLE_YAML
+
+static JsonNode *
+parse_yaml_to_json (const gchar *contents,
+                    GError      **error)
+{
+  g_set_error (error, BUILDER_YAML_PARSE_ERROR, 0, "flatpak-builder was not compiled with YAML support.");
+  return NULL;
+}
+
+#endif  // FLATPAK_BUILDER_ENABLE_YAML
+
+GObject *
+builder_gobject_from_data (GType       gtype,
+                           const char *relpath,
+                           const char *contents,
+                           GError    **error)
+{
+  if (g_str_has_suffix (relpath, ".yaml") || g_str_has_suffix (relpath, ".yml"))
+    {
+      g_autoptr(JsonNode) json = parse_yaml_to_json (contents, error);
+      if (json != NULL)
+        return json_gobject_deserialize (gtype, json);
+      else
+        return NULL;
+    }
+  else
+    return json_gobject_from_data (gtype, contents, -1, error);
+}
 
 /*
  * This code is based on debugedit.c from rpm, which has this copyright:
@@ -1430,7 +1589,7 @@ sigterm_handler (gpointer user_data)
                                G_DBUS_CALL_FLAGS_NONE, -1,
                                NULL, NULL);
 
-  kill (getpid (), SIGTERM);
+  kill (getpid (), SIGKILL);
   return TRUE;
 }
 
@@ -1449,7 +1608,7 @@ sigint_handler (gpointer user_data)
                                G_DBUS_CALL_FLAGS_NONE, -1,
                                NULL, NULL);
 
-  kill (getpid (), SIGTERM);
+  kill (getpid (), SIGKILL);
   return TRUE;
 }
 
@@ -1474,9 +1633,19 @@ builder_host_spawnv (GFile                *dir,
   guint sigterm_id = 0, sigint_id = 0;
   g_autofree gchar *commandline = NULL;
   g_autoptr(GOutputStream) out = NULL;
+  g_autoptr(GFile) cwd = NULL;
+  glnx_fd_close int blocking_stdin_fd = -1;
   int pipefd[2];
+  int stdin_fd;
   int i;
 
+  if (dir == NULL)
+    {
+      g_autofree char *current_dir = g_get_current_dir ();
+      cwd = g_file_new_for_path (current_dir);
+      dir = cwd;
+    }
+
   commandline = flatpak_quote_argv ((const char **) argv);
   g_debug ("Running '%s' on host", commandline);
 
@@ -1499,7 +1668,20 @@ builder_host_spawnv (GFile                *dir,
                                                      host_command_exited_cb,
                                                      &data, NULL);
 
-  stdin_handle = g_unix_fd_list_append (fd_list, 0, error);
+  if ((flags & G_SUBPROCESS_FLAGS_STDIN_INHERIT) != 0)
+    stdin_fd = 0;
+  else
+    {
+      blocking_stdin_fd = open ("/dev/null", O_RDONLY| O_CLOEXEC);
+      if (blocking_stdin_fd == -1)
+        {
+          glnx_set_error_from_errno (error);
+          return FALSE;
+        }
+      stdin_fd = blocking_stdin_fd;
+    }
+
+  stdin_handle = g_unix_fd_list_append (fd_list, stdin_fd, error);
   if (stdin_handle == -1)
     return FALSE;
 
@@ -1550,8 +1732,10 @@ builder_host_spawnv (GFile                *dir,
   for (i = 0; env_vars[i] != NULL; i++)
     {
       const char *env_var = env_vars[i];
-      g_variant_builder_add (env_builder, "{ss}", env_var, g_getenv (env_var));
+      if (strcmp (env_var, "LANGUAGE") != 0)
+        g_variant_builder_add (env_builder, "{ss}", env_var, g_getenv (env_var));
     }
+  g_variant_builder_add (env_builder, "{ss}", "LANGUAGE", "C");
 
   sigterm_id = g_unix_signal_add (SIGTERM, sigterm_handler, &data);
   sigint_id = g_unix_signal_add (SIGINT, sigint_handler, &data);
@@ -1620,7 +1804,7 @@ builder_maybe_host_spawnv (GFile                *dir,
                            const gchar * const  *argv)
 {
   if (flatpak_is_in_sandbox ())
-    return builder_host_spawnv (dir, output, 0, error, argv);
+    return builder_host_spawnv (dir, output, flags, error, argv);
 
   return flatpak_spawnv (dir, output, flags, error, argv);
 }
@@ -1711,10 +1895,11 @@ compare_checksum (const char *name,
   return TRUE;
 }
 
+#define GET_BUFFER_SIZE 8192
+
 gboolean
 builder_verify_checksums (const char *name,
-                          const char *data,
-                          gsize len,
+                          GFile *file,
                           const char *checksums[BUILDER_CHECKSUMS_LEN],
                           GChecksumType checksums_type[BUILDER_CHECKSUMS_LEN],
                           GError **error)
@@ -1723,9 +1908,42 @@ builder_verify_checksums (const char *name,
 
   for (i = 0; checksums[i] != NULL; i++)
     {
-      g_autofree char *checksum = NULL;
-      checksum = g_compute_checksum_for_string (checksums_type[i], data, len);
-      if (!compare_checksum (name, checksums[i], checksums_type[i], checksum, error))
+      g_autoptr(GFileInputStream) stream = NULL;
+
+      stream = g_file_read (file, NULL, error);
+
+      if (stream == NULL)
+        return FALSE;
+
+      gssize bytes_read;
+      guchar buffer[GET_BUFFER_SIZE];
+      GChecksum *checksum = NULL;
+      const char *checksum_string = NULL;
+      gboolean is_valid;
+
+      checksum = g_checksum_new (checksums_type[i]);
+
+      while ((bytes_read = g_input_stream_read (G_INPUT_STREAM(stream),
+                                                buffer, GET_BUFFER_SIZE,
+                                                NULL, error)) > 0)
+        {
+          g_checksum_update (checksum, buffer, bytes_read);
+        }
+
+      if (bytes_read < 0)
+        {
+          is_valid = FALSE;
+        }
+      else
+        {
+          checksum_string = g_checksum_get_string (checksum);
+          is_valid = compare_checksum (name, checksums[i], checksums_type[i],
+                                       checksum_string, error);
+        }
+
+      g_checksum_free (checksum);
+
+      if (!is_valid)
         return FALSE;
     }
 
@@ -1733,29 +1951,62 @@ builder_verify_checksums (const char *name,
 }
 
 typedef struct {
-  int stage;
-  gboolean printed_something;
-} DownloadPromptData;
+  GOutputStream  *out;
+  GChecksum     **checksums;
+  gsize           n_checksums;
+  GError        **error;
+} CURLWriteData;
 
-static void
-download_progress (guint64 downloaded_bytes,
-                   gpointer user_data)
+static gsize
+builder_curl_write_cb (gpointer *buffer,
+                       gsize     size,
+                       gsize     nmemb,
+                       gpointer *userdata)
 {
-  DownloadPromptData *progress_data = user_data;
-  char chrs[] = { '|', '/', '-', '\\' };
+  gsize bytes_written;
+  CURLWriteData *write_data = (CURLWriteData *) userdata;
 
-  if (progress_data->printed_something)
-    g_print ("\b");
-  progress_data->printed_something = TRUE;
-  g_print ("%c", chrs[progress_data->stage]);
-  progress_data->stage =  (progress_data->stage + 1) % G_N_ELEMENTS(chrs);
+  flatpak_write_update_checksum (write_data->out, buffer, size * nmemb, &bytes_written,
+                                 write_data->checksums, write_data->n_checksums,
+                                 NULL, write_data->error);
+
+  return bytes_written;
 }
 
-static void
-download_progress_cleanup (DownloadPromptData *progress_data)
+static gboolean
+builder_download_uri_curl (SoupURI        *uri,
+                           CURL           *session,
+                           GOutputStream  *out,
+                           GChecksum     **checksums,
+                           gsize           n_checksums,
+                           GError        **error)
 {
-  if (progress_data->printed_something)
-    g_print ("\b");
+  CURLcode retcode;
+  CURLWriteData write_data;
+  static gchar error_buffer[CURL_ERROR_SIZE];
+  g_autofree gchar *url = soup_uri_to_string (uri, FALSE);
+
+  curl_easy_setopt (session, CURLOPT_URL, url);
+  curl_easy_setopt (session, CURLOPT_WRITEFUNCTION, builder_curl_write_cb);
+  curl_easy_setopt (session, CURLOPT_WRITEDATA, &write_data);
+  curl_easy_setopt (session, CURLOPT_ERRORBUFFER, error_buffer);
+
+  write_data.out = out;
+  write_data.checksums = checksums;
+  write_data.n_checksums = n_checksums;
+  write_data.error = error;
+
+  *error_buffer = '\0';
+  retcode = curl_easy_perform (session);
+
+  if (retcode != CURLE_OK)
+    {
+      g_set_error_literal (error, BUILDER_CURL_ERROR, retcode,
+                           *error_buffer ? error_buffer : curl_easy_strerror (retcode));
+      return FALSE;
+    }
+
+  return TRUE;
 }
 
 gboolean
@@ -1763,16 +2014,13 @@ builder_download_uri (SoupURI        *uri,
                       GFile          *dest,
                       const char     *checksums[BUILDER_CHECKSUMS_LEN],
                       GChecksumType   checksums_type[BUILDER_CHECKSUMS_LEN],
-                      SoupSession    *session,
+                      CURL           *curl_session,
                       GError        **error)
 {
-  g_autoptr(SoupRequest) req = NULL;
-  g_autoptr(GInputStream) input = NULL;
   g_autoptr(GFileOutputStream) out = NULL;
   g_autoptr(GFile) tmp = NULL;
   g_autoptr(GFile) dir = NULL;
   g_autoptr(GPtrArray) checksum_array = g_ptr_array_new_with_free_func ((GDestroyNotify)g_checksum_free);
-  DownloadPromptData progress_data = {0};
   g_autofree char *basename = g_file_get_basename (dest);
   g_autofree char *template = g_strconcat (".", basename, "XXXXXX", NULL);
   gsize i;
@@ -1793,27 +2041,12 @@ builder_download_uri (SoupURI        *uri,
   if (out == NULL)
     return FALSE;
 
-  req = soup_session_request_uri (session, uri, error);
-  if (req == NULL)
-    return FALSE;
-
-  input = soup_request_send (req, NULL, error);
-  if (input == NULL)
-    return FALSE;
-
-  if (SOUP_IS_REQUEST_HTTP (req))
-    {
-      g_autoptr(SoupMessage) msg = soup_request_http_get_message (SOUP_REQUEST_HTTP(req));
-      if (!SOUP_STATUS_IS_SUCCESSFUL(msg->status_code))
-        {
-          g_set_error_literal (error, SOUP_HTTP_ERROR, msg->status_code, msg->reason_phrase);
-          return FALSE;
-        }
-    }
-
-  if (!flatpak_splice_update_checksum (G_OUTPUT_STREAM (out), input,
-                                       (GChecksum **)checksum_array->pdata, checksum_array->len,
-                                       download_progress, &progress_data, NULL, error))
+  if (!builder_download_uri_curl (uri,
+                                  curl_session,
+                                  G_OUTPUT_STREAM (out),
+                                  (GChecksum **)checksum_array->pdata,
+                                  checksum_array->len,
+                                  error))
     {
       unlink (flatpak_file_get_path_cached (tmp));
       return FALSE;
@@ -1826,8 +2059,6 @@ builder_download_uri (SoupURI        *uri,
       return FALSE;
     }
 
-  download_progress_cleanup (&progress_data);
-
   for (i = 0; checksums[i] != NULL; i++)
     {
       const char *checksum = g_checksum_get_string (g_ptr_array_index (checksum_array, i));
@@ -1847,17 +2078,182 @@ builder_download_uri (SoupURI        *uri,
   return TRUE;
 }
 
+typedef struct {
+  GParamSpec *pspec;
+  JsonNode *data;
+} BuilderXProperty;
+
+static BuilderXProperty *
+builder_x_property_new (const char *name)
+{
+  BuilderXProperty *property = g_new0 (BuilderXProperty, 1);
+  property->pspec = g_param_spec_boxed (name, "", "", JSON_TYPE_NODE, G_PARAM_READWRITE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB);
+
+  return property;
+}
+
+static void
+builder_x_property_free (BuilderXProperty *prop)
+{
+  g_param_spec_unref (prop->pspec);
+  if (prop->data)
+    json_node_unref (prop->data);
+  g_free (prop);
+}
+
+static const char *
+builder_x_property_get_name (BuilderXProperty *prop)
+{
+  return g_param_spec_get_name (prop->pspec);
+}
+
 GParamSpec *
-builder_serializable_find_property_with_error (JsonSerializable *serializable,
-                                               const char       *name)
+builder_serializable_find_property (JsonSerializable *serializable,
+                                    const char       *name)
 {
   GParamSpec *pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (serializable), name);
+
   if (pspec == NULL &&
-      !g_str_has_prefix (name, "x-"))
+      g_str_has_prefix (name, "x-"))
+    {
+      GHashTable *x_props = g_object_get_data (G_OBJECT (serializable), "flatpak-x-props");
+      BuilderXProperty *prop;
+
+      if (x_props == NULL)
+        {
+          x_props = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, (GDestroyNotify)builder_x_property_free);
+          g_object_set_data_full (G_OBJECT (serializable), "flatpak-x-props", x_props, (GDestroyNotify)g_hash_table_unref);
+        }
+
+      prop = g_hash_table_lookup (x_props, name);
+      if (prop == NULL)
+        {
+          prop = builder_x_property_new (name);
+          g_hash_table_insert (x_props, (char *)builder_x_property_get_name (prop), prop);
+        }
+
+      pspec = prop->pspec;
+    }
+
+  if (pspec == NULL &&
+      !g_str_has_prefix (name, "__") &&
+      !g_str_has_prefix (name, "//"))
     g_warning ("Unknown property %s for type %s", name, g_type_name_from_instance ((GTypeInstance *)serializable));
+
   return pspec;
 }
 
+GParamSpec **
+builder_serializable_list_properties (JsonSerializable *serializable,
+                                      guint            *n_pspecs)
+{
+  GPtrArray *res = g_ptr_array_new ();
+  guint n_normal, i;
+  g_autofree GParamSpec **normal = NULL;
+  GHashTable *x_props;
+
+  normal = g_object_class_list_properties (G_OBJECT_GET_CLASS (serializable), &n_normal);
+
+  for (i = 0; i < n_normal; i++)
+    g_ptr_array_add (res, normal[i]);
+
+  x_props = g_object_get_data (G_OBJECT (serializable), "flatpak-x-props");
+  if (x_props)
+    {
+      GLNX_HASH_TABLE_FOREACH_V (x_props, BuilderXProperty *, prop)
+        {
+          g_ptr_array_add (res, prop->pspec);
+        }
+    }
+
+  if (n_pspecs)
+    *n_pspecs = res->len;
+
+  g_ptr_array_add (res, NULL);
+
+  return (GParamSpec **)g_ptr_array_free (res, FALSE);
+}
+
+gboolean
+builder_serializable_deserialize_property (JsonSerializable *serializable,
+                                           const gchar      *property_name,
+                                           GValue           *value,
+                                           GParamSpec       *pspec,
+                                           JsonNode         *property_node)
+{
+  GHashTable *x_props = g_object_get_data (G_OBJECT (serializable), "flatpak-x-props");
+
+  if (x_props)
+    {
+      BuilderXProperty *prop = g_hash_table_lookup (x_props, property_name);
+      if (prop)
+        {
+          g_value_set_boxed (value, property_node);
+          return TRUE;
+        }
+    }
+
+  return json_serializable_default_deserialize_property (serializable, property_name, value, pspec, property_node);
+}
+
+JsonNode *
+builder_serializable_serialize_property (JsonSerializable *serializable,
+                                         const gchar      *property_name,
+                                         const GValue     *value,
+                                         GParamSpec       *pspec)
+{
+  GHashTable *x_props = g_object_get_data (G_OBJECT (serializable), "flatpak-x-props");
+
+  if (x_props)
+    {
+      BuilderXProperty *prop = g_hash_table_lookup (x_props, property_name);
+      if (prop)
+        return g_value_dup_boxed (value);
+    }
+
+  return json_serializable_default_serialize_property (serializable, property_name, value, pspec);
+}
+
+void
+builder_serializable_set_property (JsonSerializable *serializable,
+                                   GParamSpec       *pspec,
+                                   const GValue     *value)
+{
+  GHashTable *x_props = g_object_get_data (G_OBJECT (serializable), "flatpak-x-props");
+
+  if (x_props)
+    {
+      BuilderXProperty *prop = g_hash_table_lookup (x_props, g_param_spec_get_name (pspec));
+      if (prop)
+        {
+          prop->data = g_value_dup_boxed (value);
+          return;
+        }
+    }
+
+  g_object_set_property (G_OBJECT (serializable), pspec->name, value);
+}
+
+void
+builder_serializable_get_property (JsonSerializable *serializable,
+                                   GParamSpec       *pspec,
+                                   GValue           *value)
+{
+  GHashTable *x_props = g_object_get_data (G_OBJECT (serializable), "flatpak-x-props");
+
+  if (x_props)
+    {
+      BuilderXProperty *prop = g_hash_table_lookup (x_props, g_param_spec_get_name (pspec));
+      if (prop)
+        {
+          g_value_set_boxed (value, prop->data);
+          return;
+        }
+    }
+
+  g_object_get_property (G_OBJECT (serializable), pspec->name, value);
+}
+
 void
 builder_set_term_title (const gchar *format,
                         ...)
@@ -1874,3 +2270,340 @@ builder_set_term_title (const gchar *format,
 
   g_print ("\033]2;flatpak-builder: %s\007", message);
 }
+
+typedef struct
+{
+  FlatpakXml *current;
+} XmlData;
+
+FlatpakXml *
+flatpak_xml_new (const gchar *element_name)
+{
+  FlatpakXml *node = g_new0 (FlatpakXml, 1);
+
+  node->element_name = g_strdup (element_name);
+  return node;
+}
+
+FlatpakXml *
+flatpak_xml_new_text (const gchar *text)
+{
+  FlatpakXml *node = g_new0 (FlatpakXml, 1);
+
+  node->text = g_strdup (text);
+  return node;
+}
+
+void
+flatpak_xml_add (FlatpakXml *parent, FlatpakXml *node)
+{
+  node->parent = parent;
+
+  if (parent->first_child == NULL)
+    parent->first_child = node;
+  else
+    parent->last_child->next_sibling = node;
+  parent->last_child = node;
+}
+
+static void
+xml_start_element (GMarkupParseContext *context,
+                   const gchar         *element_name,
+                   const gchar        **attribute_names,
+                   const gchar        **attribute_values,
+                   gpointer             user_data,
+                   GError             **error)
+{
+  XmlData *data = user_data;
+  FlatpakXml *node;
+
+  node = flatpak_xml_new (element_name);
+  node->attribute_names = g_strdupv ((char **) attribute_names);
+  node->attribute_values = g_strdupv ((char **) attribute_values);
+
+  flatpak_xml_add (data->current, node);
+  data->current = node;
+}
+
+static void
+xml_end_element (GMarkupParseContext *context,
+                 const gchar         *element_name,
+                 gpointer             user_data,
+                 GError             **error)
+{
+  XmlData *data = user_data;
+
+  data->current = data->current->parent;
+}
+
+static void
+xml_text (GMarkupParseContext *context,
+          const gchar         *text,
+          gsize                text_len,
+          gpointer             user_data,
+          GError             **error)
+{
+  XmlData *data = user_data;
+  FlatpakXml *node;
+
+  node = flatpak_xml_new (NULL);
+  node->text = g_strndup (text, text_len);
+  flatpak_xml_add (data->current, node);
+}
+
+static void
+xml_passthrough (GMarkupParseContext *context,
+                 const gchar         *passthrough_text,
+                 gsize                text_len,
+                 gpointer             user_data,
+                 GError             **error)
+{
+}
+
+static GMarkupParser xml_parser = {
+  xml_start_element,
+  xml_end_element,
+  xml_text,
+  xml_passthrough,
+  NULL
+};
+
+void
+flatpak_xml_free (FlatpakXml *node)
+{
+  FlatpakXml *child;
+
+  if (node == NULL)
+    return;
+
+  child = node->first_child;
+  while (child != NULL)
+    {
+      FlatpakXml *next = child->next_sibling;
+      flatpak_xml_free (child);
+      child = next;
+    }
+
+  g_free (node->element_name);
+  g_free (node->text);
+  g_strfreev (node->attribute_names);
+  g_strfreev (node->attribute_values);
+  g_free (node);
+}
+
+
+void
+flatpak_xml_to_string (FlatpakXml *node, GString *res)
+{
+  int i;
+  FlatpakXml *child;
+
+  if (node->parent == NULL)
+    g_string_append (res, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+
+  if (node->element_name)
+    {
+      if (node->parent != NULL)
+        {
+          g_string_append (res, "<");
+          g_string_append (res, node->element_name);
+          if (node->attribute_names)
+            {
+              for (i = 0; node->attribute_names[i] != NULL; i++)
+                {
+                  g_string_append_printf (res, " %s=\"%s\"",
+                                          node->attribute_names[i],
+                                          node->attribute_values[i]);
+                }
+            }
+          if (node->first_child == NULL)
+            g_string_append (res, "/>");
+          else
+            g_string_append (res, ">");
+        }
+
+      child = node->first_child;
+      while (child != NULL)
+        {
+          flatpak_xml_to_string (child, res);
+          child = child->next_sibling;
+        }
+      if (node->parent != NULL)
+        {
+          if (node->first_child != NULL)
+            g_string_append_printf (res, "</%s>", node->element_name);
+        }
+
+    }
+  else if (node->text)
+    {
+      g_autofree char *escaped = g_markup_escape_text (node->text, -1);
+      g_string_append (res, escaped);
+    }
+}
+
+FlatpakXml *
+flatpak_xml_unlink (FlatpakXml *node,
+                    FlatpakXml *prev_sibling)
+{
+  FlatpakXml *parent = node->parent;
+
+  if (parent == NULL)
+    return node;
+
+  if (parent->first_child == node)
+    parent->first_child = node->next_sibling;
+
+  if (parent->last_child == node)
+    parent->last_child = prev_sibling;
+
+  if (prev_sibling)
+    prev_sibling->next_sibling = node->next_sibling;
+
+  node->parent = NULL;
+  node->next_sibling = NULL;
+
+  return node;
+}
+
+FlatpakXml *
+flatpak_xml_find (FlatpakXml  *node,
+                  const char  *type,
+                  FlatpakXml **prev_child_out)
+{
+  FlatpakXml *child = NULL;
+  FlatpakXml *prev_child = NULL;
+
+  child = node->first_child;
+  prev_child = NULL;
+  while (child != NULL)
+    {
+      FlatpakXml *next = child->next_sibling;
+
+      if (g_strcmp0 (child->element_name, type) == 0)
+        {
+          if (prev_child_out)
+            *prev_child_out = prev_child;
+          return child;
+        }
+
+      prev_child = child;
+      child = next;
+    }
+
+  return NULL;
+}
+
+
+FlatpakXml *
+flatpak_xml_parse (GInputStream *in,
+                   gboolean      compressed,
+                   GCancellable *cancellable,
+                   GError      **error)
+{
+  g_autoptr(GInputStream) real_in = NULL;
+  g_autoptr(FlatpakXml) xml_root = NULL;
+  XmlData data = { 0 };
+  char buffer[32 * 1024];
+  gssize len;
+  g_autoptr(GMarkupParseContext) ctx = NULL;
+
+  if (compressed)
+    {
+      g_autoptr(GZlibDecompressor) decompressor = NULL;
+      decompressor = g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP);
+      real_in = g_converter_input_stream_new (in, G_CONVERTER (decompressor));
+    }
+  else
+    {
+      real_in = g_object_ref (in);
+    }
+
+  xml_root = flatpak_xml_new ("root");
+  data.current = xml_root;
+
+  ctx = g_markup_parse_context_new (&xml_parser,
+                                    G_MARKUP_PREFIX_ERROR_POSITION,
+                                    &data,
+                                    NULL);
+
+  while ((len = g_input_stream_read (real_in, buffer, sizeof (buffer),
+                                     cancellable, error)) > 0)
+    {
+      if (!g_markup_parse_context_parse (ctx, buffer, len, error))
+        return NULL;
+    }
+
+  if (len < 0)
+    return NULL;
+
+  return g_steal_pointer (&xml_root);
+}
+
+GBytes *
+flatpak_read_stream (GInputStream *in,
+                     gboolean      null_terminate,
+                     GError      **error)
+{
+  g_autoptr(GOutputStream) mem_stream = NULL;
+
+  mem_stream = g_memory_output_stream_new_resizable ();
+  if (g_output_stream_splice (mem_stream, in,
+                              0, NULL, error) < 0)
+    return NULL;
+
+  if (null_terminate)
+    {
+      if (!g_output_stream_write (G_OUTPUT_STREAM (mem_stream), "\0", 1, NULL, error))
+        return NULL;
+    }
+
+  if (!g_output_stream_close (G_OUTPUT_STREAM (mem_stream), NULL, error))
+    return NULL;
+
+  return g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (mem_stream));
+}
+
+GVariant *
+flatpak_variant_uncompress (GVariant *variant,
+                            const GVariantType *type)
+{
+  g_autoptr(GInputStream) input_stream = NULL;
+  g_autoptr(GZlibDecompressor) decompressor = NULL;
+  g_autoptr(GInputStream) converter = NULL;
+  g_autoptr(GBytes) decompressed_bytes = NULL;
+  const guint8 *compressed;
+  gsize compressed_size;
+
+  g_assert (g_variant_is_of_type (variant, G_VARIANT_TYPE_BYTESTRING));
+
+  compressed = g_variant_get_data (variant);
+  compressed_size = g_variant_get_size (variant);
+
+  input_stream = g_memory_input_stream_new_from_data (compressed, compressed_size, NULL);
+  decompressor = g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP);
+  converter = g_converter_input_stream_new (G_INPUT_STREAM (input_stream), G_CONVERTER (decompressor));
+  decompressed_bytes = flatpak_read_stream (converter, FALSE, NULL);
+  return g_variant_ref_sink (g_variant_new_from_bytes (type, decompressed_bytes, TRUE));
+}
+
+GVariant *
+flatpak_variant_compress (GVariant *variant)
+{
+  g_autoptr(GInputStream) input_stream = NULL;
+  g_autoptr(GZlibCompressor) compressor = NULL;
+  g_autoptr(GInputStream) converter = NULL;
+  g_autoptr(GBytes) compressed_bytes = NULL;
+  const guint8 *decompressed;
+  gsize decompressed_size;
+
+  decompressed = g_variant_get_data (variant);
+  decompressed_size = g_variant_get_size (variant);
+
+  input_stream = g_memory_input_stream_new_from_data (decompressed, decompressed_size, NULL);
+  compressor = g_zlib_compressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP, -1);
+  converter = g_converter_input_stream_new (G_INPUT_STREAM (input_stream), G_CONVERTER (compressor));
+  compressed_bytes = flatpak_read_stream (converter, FALSE, NULL);
+
+  return g_variant_ref_sink (g_variant_new_from_bytes (G_VARIANT_TYPE_BYTESTRING, compressed_bytes, TRUE));
+}
